<?php

namespace JReviewsDeploy\Administrator\Controller;

use Joomla\CMS\Extension\ExtensionHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\Installer\Installer;
use Joomla\CMS\Installer\InstallerHelper;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\Toolbar\ToolbarHelper;
use JReviews\App\Actions\ClearCacheAction;
use JReviews\App\Actions\InstallAddonAction;
use JReviews\App\Actions\InstallCorePackageExtrasAction;
use JReviews\App\Actions\UninstallAddonAction;
use JReviewsDeploy\Administrator\Services\AppStoreService;

defined('_JEXEC') or die;

class DisplayController extends BaseController
{
    protected $default_view = 'jreviewsdeploy';

    protected $appstore;

    protected $installer;

    protected $view;

    public function __construct(...$params)
    {
        $this->appstore = new AppStoreService();

        parent::__construct(...$params);

        $this->view = $this->getView('jreviewsdeploy', 'html', 'Administrator');

        $this->checkLicense();

        $this->checkRequirements();

        $wa = $this->app->getDocument()->getWebAssetManager();

        $wa->registerAndUseScript('htmx','https://unpkg.com/htmx.org@1.9.12');
    }

    public function display($cachable = false, $urlparams = false)
    {
        ToolbarHelper::title('JReviews Updater');

        ToolbarHelper::preferences('com_jreviewsdeploy');

        $content = $this->view->loadTemplate();

        $this->view->setContent($content);

        $this->view->setLayout('layout');

        $this->view->display();
    }

    public function startUpdate()
    {
        $installedProducts = $this->appstore->installedProducts();

        $updatable = array_filter($installedProducts, function($product) {
            if ($product['new']) {
                return true;
            }

            return false;
        });

        if (empty($updatable)) {
            $this->render();
            return;
        }

        $product = reset($updatable);

        // If it's s2framwork or JReviews, uninstall first, leave install for
        // next request so they are in completely separate requests
        if (in_array($product, $installedProducts) && in_array($product['name'], ['s2framework', 'jreviews'])) {
            $package = ExtensionHelper::getExtensionRecord('pkg_'.$product['name'], 'package');
            if ($package) {
                $this->uninstallPackage($product['name'], $package);
                $this->render();
                return;
            }
        }

        try {

            if (in_array($product['name'], ['jreviews', 's2framework'])) {
                $this->installPackage($product['name']);
            } else {
                $this->installAddon($product['name']);
            }

        } catch (\Exception $e) {

            $this->view->logError($e->getMessage());
        
        }

        $this->view->setInstalled($product['name']);

        $this->render();
    }

    protected function checkLicense()
    {
        $licenseKey = \JReviewsDeploy\jr_license();

        if (! $licenseKey) {
            $this->view->setLayout('missing_license');
            return;
        }

        try {
            if (! $this->appstore->validateLicense()) {
                $this->view->setLayout('invalid_license');
            }
        } catch (\Exception $e) {
            $this->view->setLayout('connection_error');
        }    
    }

    protected function checkRequirements()
    {
        if (version_compare(PHP_VERSION, '8.1', '<')) {
            $this->view->setLayout('failed_requirements');
        }
    }

    protected function installPackage($name)
    {
        try {
            
            $path = $this->appstore->downloadComponentPackage($name);

            if (! $path) {
                $this->view->logError('Error downloading '.$name);
                return false;
            }

        } catch (\Exception $e) {

            throw new \Exception($name.': '.$e->getMessage(), 500);
        }

        $installer = new Installer();
        
        $package = InstallerHelper::unpack($path, true);

        if (! $package['extractdir']) {
            return false;
        }
        
        $result = $installer->install($package['dir']);

        if (! $result) {
            $this->view->logError('Error installing '.$name);
            return false;
        }

        InstallerHelper::cleanupInstall($package['packagefile'], $package['extractdir']);
        
        if (class_exists(InstallCorePackageExtrasAction::class)) {
            InstallCorePackageExtrasAction::run($name);
        }

        if (class_exists(ClearCacheAction::class)) {
            ClearCacheAction::run();
        }

        return $result;
    }

    protected function uninstallPackage($name, $package)
    {
        $installer = new Installer();

        $result = $installer->uninstall('package', $package->extension_id);

        if (! $result) {
            $this->view->logError('Error uninstalling '.$name);
            return false;
        }
    }

    protected function installAddon($name)
    {
        try {
            
            $path = $this->appstore->downloadAddon($name);
                    
            if (! $path) {
                $this->view->logError('Error downloading '.$name);
                return false;
            }

            UninstallAddonAction::run($name);

            $disabledAddons = json_decode(Factory::getApplication()->input->getRaw('disabled_addons'), true);

            $enable = ! in_array($name, $disabledAddons) ? true : false;

            InstallAddonAction::run(name: $name, sourceFullPath: $path, enable: $enable);

            return true;
    
        } catch (\Exception $e) {

            throw new \Exception($name.': '.$e->getMessage(), 500);
        }
    }

    protected function render()
    {
        $this->view->display();
    }
}
