<?php

namespace JReviewsDeploy\Administrator\Services;

use Joomla\CMS\Factory;
use Joomla\CMS\Http\Http;
use Joomla\CMS\Http\HttpFactory;
use Joomla\Filesystem\Folder;
use Joomla\Http\TransportInterface;

defined('_JEXEC') or die;

class AppStoreService
{
    const API_BASE = 'https://backoffice.jreviews.com/api/appstore';

    const VALIDATE_LICENSE_ENDPOINT = 'validateLicense';

    const VERSIONS_ENDPOINT = 'versions';

    const DOWNLOAD_ENDPOINT = 'download';
    
    const DOWNLOAD_COMPONENT_PACKAGE_ENDPOINT = 'downloadComponentPackage';

    protected ?string $apiKey;

    protected string $host;
    
    protected static $cache = [];

    public function __construct()
    {
        $this->apiKey = \JReviewsDeploy\jr_license();

        $this->host = Factory::getApplication()->input->server->get('HTTP_HOST');
    }

    public function validateLicense()
    {
        if (static::$cache['validateLicense'] ?? null) {
            return static::$cache['validateLicense'];
        }

        $response = $this->connect(self::VALIDATE_LICENSE_ENDPOINT, [
            'license_key' => $this->apiKey,
        ]);

        static::$cache['validateLicense'] = $response->getStatusCode() == 200;

        return static::$cache['validateLicense'];        
    }

    public function downloadComponentPackage($name, $destination = null)
    {
        $response = $this->connect(self::DOWNLOAD_COMPONENT_PACKAGE_ENDPOINT, [
            'license_key' => $this->apiKey,
            'product'=> $name
        ]);

        if ($response->getStatusCode() !== 200) {
            return false;
        }

        if (! $destination) {

            $filepath = Factory::getApplication()->get('tmp_path').'/'.uniqid($name.'-').'.zip';
        
        } else {
        
            $filepath = $destination;

            if (! file_exists($filepath)) {
                Folder::create($filepath);
            }

            $filepath = $filepath.'/'.uniqid($name.'-').'.zip';
        }

        file_put_contents($filepath, $response->getBody()->getContents());
        
        return $filepath;
    }

    public function downloadAddon($name, $destination = null)
    {
        $filepath = null;

        $response = $this->connect(self::DOWNLOAD_ENDPOINT, [
            'license_key' => $this->apiKey,
            'product'=> $name
        ]);

        if ($response->getStatusCode() !== 200) {
            return false;
        }

        if (! $destination) {

            $filepath = Factory::getApplication()->get('tmp_path').'/'.uniqid($name.'-').'.zip';
        
        } else {
        
            $filepath = $destination;

            if (! file_exists($filepath)) {
                Folder::create($filepath);
            }

            $filepath = $filepath.'/'.uniqid($name.'-').'.zip';
        }

        file_put_contents($filepath, $response->getBody()->getContents());

        return $filepath;
    }

    public function latestVersions()
    {
        if (static::$cache['latestVersions'] ?? null) {
            return static::$cache['latestVersions'];
        }

        $response = $this->connect(self::VERSIONS_ENDPOINT);

        if ($response->getStatusCode() !== 200) {
            return false;
        }

        static::$cache['latestVersions'] = json_decode($response->getBody()->getContents(), true);

        return static::$cache['latestVersions'];
    }    

    public function installedProducts()
    {
        $installed = array_merge([
            (object) [
                'name' => 's2framework',
                'title' => 's2framework',
                'version' => \JReviewsDeploy\component_version('com_s2framework')
            ],
            (object) [
                'name' => 'jreviews',
                'title' => 'JReviews',
                'version' => \JReviewsDeploy\component_version('com_jreviews')
            ],
        ], \JReviewsDeploy\jr_addons());

        $available = $this->latestVersions();

        $products = [];

        foreach ($installed as $i) {
            if (! isset($available[$i->name])) {
                continue;
            }

            if ($i->version == '1.0.5' || version_compare($i->version ?? 0, $available[$i->name], '<')) {
                $products[] = [
                    'title' => $i->title,
                    'name' => $i->name,
                    'current' => $i->version,
                    'new' => $available[$i->name]
                ];
            } else {
                $products[] = [
                    'title' => $i->title,
                    'name' => $i->name,
                    'current' => $i->version,
                    'new' => null,
                ];
            }
        }

        return $products;
    }
        
    protected function connect($endpoint, $payload = [])
    {
        $endpoint = static::API_BASE."/$endpoint";

        $installedVersions = $this->installedVersions();

        $payload = array_merge($payload, [
            'platform'      => 'joomla',
            'host'          => $this->host,
            'versions'      => $installedVersions,
            'php_version'   => PHP_VERSION
        ]);

        $transport = (new HttpFactory())->getAvailableDriver([
            'transport.curl' => [
                CURLOPT_SSL_VERIFYPEER => false
            ]
        ]);

        // Ensure the transport is a TransportInterface instance or bail out
        if (! ($transport instanceof TransportInterface)) {
            throw new \InvalidArgumentException(sprintf('A valid %s object was not set.', TransportInterface::class));
        }

        $http = new Http([], $transport);

        $response = $http->post($endpoint, $payload);

        $response->getBody()->rewind();
    
        return $response;
    }     

    public function installedVersions()
    {
        $addons = \JReviewsDeploy\jr_addons_array();

        $jreviews = \JReviewsDeploy\component_version('com_jreviews');

        $framework = \JReviewsDeploy\component_version('com_s2framework');
        
        $jreviews = $jreviews == '[VERSION]' ? \JReviewsDeploy\jr_config('jreviews-version') : $jreviews;

        $framework = $framework == '[VERSION]' ? \JReviewsDeploy\jr_config('s2framework-version') : $framework;
        
        return [
            's2framework'   => $framework,
            'jreviews'      => $jreviews,
        ] + $addons;        
    }    
}